
import uuid
import time


from NVDAObjects.window.excel import ExcelCell, ExcelWorksheet
import eventHandler
import ui
from logHandler import log


licenseDenied = False

class ExcelCellOverlay(ExcelCell):

	def goBackToCellBeforeMonitorCell(self):
		global licenseDenied
		if licenseDenied:
			raise licenseError
		# get last cell from monitorCellTracker
		lastCell = self.parent.monitorCellTracker.getLastCell()
		if lastCell:
			lastCellObject = self.parent.excelWorksheetObject.cells(lastCell[0], lastCell[1])
			lastCellObject.select()
			lastCellObject.activate()

			cellObj = ExcelCellOverlay(windowHandle=self.windowHandle, excelWindowObject=self.excelWindowObject,
									   excelCellObject=lastCellObject)
			eventHandler.executeEvent('gainFocus', cellObj)

	def dialogReturned(self, returnedValue):
		global licenseDenied
		if licenseDenied:
			raise licenseError
		isRowHeader, isColumnHeader = returnedValue
		maxRowNumber = 0
		maxColumnNumber = 0

		# the previous info is handled as default
		oldInfo = self.parent.headerCellTracker.getHeaderCellInfoAt(self.rowNumber, self.columnNumber)
		if oldInfo:
			if isColumnHeader and not oldInfo.isColumnHeader:
				oldInfo.isColumnHeader = True
				oldInfo.rowSpan = self.rowSpan
			elif isRowHeader and not oldInfo.isRowHeader:
				oldInfo.isRowHeader = True
				oldInfo.colSpan = self.colSpan
			else:
				return False
			isColumnHeader = oldInfo.isColumnHeader
			isRowHeader = oldInfo.isRowHeader

		if isColumnHeader and isRowHeader:
			name = "Title_"
		elif isRowHeader:
			name = "RowTitle_"
		elif isColumnHeader:
			name = "ColumnTitle_"
		else:
			raise ValueError("One or both of isColumnHeader or isRowHeader must be True")

		if not maxRowNumber == None:
			name += "." + str(maxRowNumber)
		if not maxColumnNumber == None:
			name += "." + str(maxColumnNumber)
		# if we set max row and max col as 0, we want it to be undefined so that it covers the whole sheet.
		if maxRowNumber == 0: maxRowNumber = None
		if maxColumnNumber == 0: maxColumnNumber = None

		# this is done as default to make the name unique, the name id is added for potential future use
		name += '.' + uuid.uuid4().hex
		relativeName = name
		name = "%s!%s" % (self.excelRangeObject.worksheet.name, name)

		if oldInfo:
			self.parent.excelWorksheetObject.parent.names(oldInfo.name).delete()
			oldInfo.name = name
		self.parent.headerCellTracker.addHeaderCellInfo(rowNumber=self.rowNumber,
														columnNumber=self.columnNumber,
														rowSpan=self.rowSpan,
														colSpan=self.colSpan,
														maxRowNumber=maxRowNumber,
														maxColumnNumber=maxColumnNumber,
														name=name,
														isColumnHeader=isColumnHeader,
														isRowHeader=isRowHeader)

		self.parent.excelWorksheetObject.names.add(relativeName, self.excelRangeObject)

	def goToMonitorCell(self, monitorCell):
		global licenseDenied
		if licenseDenied:
			raise licenseError
		monitorCellObject = self.parent.excelWorksheetObject.cells(monitorCell[0], monitorCell[1])
		monitorCellObject.select()
		monitorCellObject.activate()

		cellObj = ExcelCellOverlay(windowHandle=self.windowHandle, excelWindowObject=self.excelWindowObject,
								   excelCellObject=monitorCellObject)
		eventHandler.executeEvent('gainFocus', cellObj)


class ExcelWorksheetOverlay(ExcelWorksheet):


	def setAsMonitorCellBase(self, cell, monitorIndex, sentences):
		global licenseDenied
		if licenseDenied:
			raise licenseError
		""" Based on setAsHeaderCell """
		oldInfo = self.monitorCellTracker.getMonitorCellInfoAt(cell.rowNumber, cell.columnNumber)
		if oldInfo:
			# cell is already monitor cell
			ui.message(sentences[0].format(cellCoords=cell._get_cellCoordsText()))
			return False
		elif self.monitorCellTracker.getMonitorCellInfoAtIndex(monitorIndex):
			# monitor index already in use
			ui.message(sentences[1].format(monitorIndex=monitorIndex))
			return False

		name = "Monitor_" + str(monitorIndex)
		name += uuid.uuid4().hex
		relativeName = name
		name = "%s!%s" % (cell.excelRangeObject.worksheet.name, name)

		self.monitorCellTracker.addMonitorCellInfo(rowNumber=cell.rowNumber, columnNumber=cell.columnNumber, name=name, index=monitorIndex)
		self.excelWorksheetObject.names.add(relativeName, cell.excelRangeObject)
		ui.message(sentences[2].format(cellCoords=cell._get_cellCoordsText(), monitorIndex=monitorIndex))
		return True


def hasLocalLicense():
	log.info("checking local license")
	try:
		import config
		if config.conf.get("excelNavProLicense") == "39U4isWcuQiqLpsx4Ype":
			return True
		return False
	except:
		return False


try:
	import addonHandler
	import wx
	addon: addonHandler.Addon = addonHandler.getCodeAddon()
	licenseExtension = addon.loadModule("lib.licenseExtension")
	licenseExtension.isLicensed()
	def checkLicense():

		global licenseDenied
		res = licenseExtension.isLicensed()
		if not res:
			if not hasLocalLicense():
				licenseDenied = True

	checkLicense()
	wx.CallLater(500, checkLicense)
	wx.CallLater(5000, checkLicense)
except Exception as e:
	log.error(f"could not load licenseExtension module {e}")




class licenseError(Exception):
	pass











