

import addonHandler
from comtypes import COMError
from NVDAObjects.window import excel
from logHandler import log

addonHandler.initTranslation()

re_definedName = excel.ExcelWorksheet.re_definedName

def collectMonitorCellsFromWorksheet(excelWorksheetObject):
    """
    This function collects all monitor cells from a excelWorksheetObject,
    checks wheter the name starts with "monitor_" and takes the information form that cell
    """
    sheetName = excelWorksheetObject.name
    monitorList = []
    for x in excelWorksheetObject.parent.names:
        fullName = x.name
        nameMatch = re_definedName.match(fullName)
        if not nameMatch:
            continue
        sheet = nameMatch.group('sheet')
        if sheet and sheet[0] == "'" and sheet[-1] == "'":
            sheet = sheet[1:-1]
        if sheet and sheet != sheetName:
            continue
        name = nameMatch.group('name').lower()
        isMonitorCell = False
        if name.startswith('monitor_'):

            isMonitorCell = True
            monitorIndex = int(name[len('monitor_')])
        else:
            continue
        try:
            monitorCell = x.refersToRange
        except COMError:
            continue
        if monitorCell.parent.name != sheetName:
            continue

        monitorTuple = (monitorCell.row, monitorCell.column, monitorIndex, fullName)
        monitorList.append(monitorTuple)
    return monitorList

class monitorCellInfo(object):
    """ Cell info template for monitor cells. Based On HeaderCellInfo."""
    __slots__ = ['rowNumber', 'columnNumber', 'name', 'index']

    def __init__(self, **kwargs):
        for name, value in kwargs.items():
            setattr(self, name, value)


lastCell = None

class MonitorCellTracker(object):
    """
    Based on HeaderCellTracker in tableUtils. Keeps track of all monitor cells and are stored in infosDict. Can keep
    Track of 10 monitor cells listed in a dictionary infosDict as well as a list with the correct index.
    """

    def __init__(self):
        self.infosDict = {}
        self.DictByIndex = {}
        self.lastCell = None

    def addMonitorCellInfo(self, **kwargs):
        info = monitorCellInfo(**kwargs)
        key = (info.rowNumber, info.columnNumber)
        self.infosDict[key] = info
        self.DictByIndex[info.index] = key

    def removeMonitorCellInfo(self, info):
        key = (info.rowNumber, info.columnNumber)
        del self.infosDict[key]
        del self.DictByIndex[info.index]

    def getMonitorCellInfoAt(self, rowNumber, columnNumber):
        return self.infosDict.get((rowNumber, columnNumber))

    def getMonitorCellInfoAtIndex(self, index):
        return self.DictByIndex.get(index)

    def getAllMonitorCells(self):
        return list(self.infosDict.keys())

    def setLastCell(self, row, col ):
        # lastCell is global since MonitorCellTracker is refreshed every time a focus changed and we do not want
        # this in the name of a cell in the sheet itself ( like regular monitor cells )
        global lastCell
        lastCell = (row, col)

    def getLastCell(self):
        global lastCell
        toBeReturned =  lastCell
        lastCell = None
        return toBeReturned
