

import speech.speech
from speech.speech import *
from speech.speech import _speechState
from logHandler import log
import globalPluginHandler


# C901 'getPropertiesSpeech' is too complex
# Note: when working on getPropertiesSpeech, look for opportunities to simplify
# and move logic out into smaller helper functions.
def getPropertiesSpeechExcel(  # noqa: C901
		reason: OutputReason = OutputReason.QUERY,
		**propertyValues
) -> SpeechSequence:
	textList: List[str] = []
	name: Optional[str] = propertyValues.get('name')
	if name:
		textList.append(name)
	if 'role' in propertyValues:
		role: controlTypes.Role = propertyValues['role']
		speakRole=True
	elif '_role' in propertyValues:
		speakRole=False
		role: controlTypes.Role = propertyValues['_role']
	else:
		speakRole=False
		role=controlTypes.Role.UNKNOWN
	role = controlTypes.Role(role)
	value: Optional[str] = propertyValues.get('value') if role not in controlTypes.silentValuesForRoles else None
	cellCoordsText: Optional[str] = propertyValues.get('cellCoordsText')
	rowNumber = propertyValues.get('rowNumber')
	columnNumber = propertyValues.get('columnNumber')
	includeTableCellCoords = propertyValues.get('includeTableCellCoords', True)

	### moving this to the front ###
	if cellCoordsText or rowNumber or columnNumber:
		tableID = propertyValues.get("_tableID")
		# Always treat the table as different if there is no tableID.
		sameTable = (tableID and tableID == _speechState.oldTableID)
		# Don't update the oldTableID if no tableID was given.
		if tableID and not sameTable:
			_speechState.oldTableID = tableID
		# When fetching row and column span
		# default the values to 1 to make further checks a lot simpler.
		# After all, a table cell that has no rowspan implemented is assumed to span one row.
		rowSpan = propertyValues.get("rowSpan") or 1
		columnSpan = propertyValues.get("columnSpan") or 1
		if rowNumber and (
			not sameTable
			or rowNumber != _speechState.oldRowNumber
			or rowSpan != _speechState.oldRowSpan
		):
			rowHeaderText: Optional[str] = propertyValues.get("rowHeaderText")
			if rowHeaderText:
				textList.append(rowHeaderText)
			if includeTableCellCoords and not cellCoordsText:
				# Translators: Speaks current row number (example output: row 3).
				rowNumberTranslation: str = _("row %s") % rowNumber
				textList.append(rowNumberTranslation)
				if rowSpan>1 and columnSpan<=1:
					# Translators: Speaks the row span added to the current row number (example output: through 5).
					rowSpanAddedTranslation: str = _("through %s") % (rowNumber + rowSpan - 1)
					textList.append(rowSpanAddedTranslation)
			_speechState.oldRowNumber = rowNumber
			_speechState.oldRowSpan = rowSpan
		if columnNumber and (
			not sameTable
			or columnNumber != _speechState.oldColumnNumber
			or columnSpan != _speechState.oldColumnSpan
		):
			columnHeaderText: Optional[str] = propertyValues.get("columnHeaderText")
			if columnHeaderText:
				textList.append(columnHeaderText)
			if includeTableCellCoords and not cellCoordsText:
				# Translators: Speaks current column number (example output: column 3).
				colNumberTranslation: str = _("column %s") % columnNumber
				textList.append(colNumberTranslation)
				if columnSpan>1 and rowSpan<=1:
					# Translators: Speaks the column span added to the current column number (example output: through 5).
					colSpanAddedTranslation: str = _("through %s") % (columnNumber + columnSpan - 1)
					textList.append(colSpanAddedTranslation)
			_speechState.oldColumnNumber = columnNumber
			_speechState.oldColumnSpan = columnSpan
		if includeTableCellCoords and not cellCoordsText and rowSpan>1 and columnSpan>1:
			# Translators: Speaks the row and column span added to the current row and column numbers
			#			(example output: through row 5 column 3).
			rowColSpanTranslation: str = _("through row {row} column {column}").format(
				row=rowNumber + rowSpan - 1,
				column=columnNumber + columnSpan - 1
			)
			textList.append(rowColSpanTranslation)
	### moving this to the front ###

	if role == controlTypes.Role.CHARTELEMENT:
		speakRole = False
	roleText: Optional[str] = propertyValues.get('roleText')
	if (
		speakRole
		and (
			roleText
			or reason not in (
				OutputReason.SAYALL,
				OutputReason.CARET,
				OutputReason.FOCUS,
				OutputReason.QUICKNAV
			)
			or not (
				name
				or value
				or cellCoordsText
				or rowNumber
				or columnNumber
			)
			or role not in controlTypes.silentRolesOnFocus
		)
		and (
			role != controlTypes.Role.MATH
			or reason not in (
				OutputReason.CARET,
				OutputReason.SAYALL
			)
	)):
		textList.append(roleText if roleText else role.displayString)


	if value:
		textList.append(value)
	states = propertyValues.get('states')
	realStates=propertyValues.get('_states',states)
	negativeStates=propertyValues.get('negativeStates',set())
	# If the caller didn't want states, states will be None.
	# However, empty states means the caller still wants states, but the object
	# had no states; e.g. an unchecked check box with no other states.
	if states is not None or negativeStates:
		if states is None:
			# processAndLabelStates won't accept None for states.
			states = set()
		labelStates = controlTypes.processAndLabelStates(role, realStates, reason, states, negativeStates)
		textList.extend(labelStates)
	# sometimes description key is present but value is None
	description: Optional[str] = propertyValues.get('description')
	if description:
		textList.append(description)
	# sometimes keyboardShortcut key is present but value is None
	keyboardShortcut: Optional[str] = propertyValues.get('keyboardShortcut')
	if keyboardShortcut:
		textList.append(keyboardShortcut)
	if includeTableCellCoords and cellCoordsText:
		textList.append(cellCoordsText)
	# if cellCoordsText or rowNumber or columnNumber:
	# 	tableID = propertyValues.get("_tableID")
	# 	# Always treat the table as different if there is no tableID.
	# 	sameTable = (tableID and tableID == _speechState.oldTableID)
	# 	# Don't update the oldTableID if no tableID was given.
	# 	if tableID and not sameTable:
	# 		_speechState.oldTableID = tableID
	# 	# When fetching row and column span
	# 	# default the values to 1 to make further checks a lot simpler.
	# 	# After all, a table cell that has no rowspan implemented is assumed to span one row.
	# 	rowSpan = propertyValues.get("rowSpan") or 1
	# 	columnSpan = propertyValues.get("columnSpan") or 1
	# 	if rowNumber and (
	# 		not sameTable
	# 		or rowNumber != _speechState.oldRowNumber
	# 		or rowSpan != _speechState.oldRowSpan
	# 	):
	# 		rowHeaderText: Optional[str] = propertyValues.get("rowHeaderText")
	# 		if rowHeaderText:
	# 			textList.append(rowHeaderText)
	# 		if includeTableCellCoords and not cellCoordsText:
	# 			# Translators: Speaks current row number (example output: row 3).
	# 			rowNumberTranslation: str = _("row %s") % rowNumber
	# 			textList.append(rowNumberTranslation)
	# 			if rowSpan>1 and columnSpan<=1:
	# 				# Translators: Speaks the row span added to the current row number (example output: through 5).
	# 				rowSpanAddedTranslation: str = _("through %s") % (rowNumber + rowSpan - 1)
	# 				textList.append(rowSpanAddedTranslation)
	# 		_speechState.oldRowNumber = rowNumber
	# 		_speechState.oldRowSpan = rowSpan
	# 	if columnNumber and (
	# 		not sameTable
	# 		or columnNumber != _speechState.oldColumnNumber
	# 		or columnSpan != _speechState.oldColumnSpan
	# 	):
	# 		columnHeaderText: Optional[str] = propertyValues.get("columnHeaderText")
	# 		if columnHeaderText:
	# 			textList.append(columnHeaderText)
	# 		if includeTableCellCoords and not cellCoordsText:
	# 			# Translators: Speaks current column number (example output: column 3).
	# 			colNumberTranslation: str = _("column %s") % columnNumber
	# 			textList.append(colNumberTranslation)
	# 			if columnSpan>1 and rowSpan<=1:
	# 				# Translators: Speaks the column span added to the current column number (example output: through 5).
	# 				colSpanAddedTranslation: str = _("through %s") % (columnNumber + columnSpan - 1)
	# 				textList.append(colSpanAddedTranslation)
	# 		_speechState.oldColumnNumber = columnNumber
	# 		_speechState.oldColumnSpan = columnSpan
	# 	if includeTableCellCoords and not cellCoordsText and rowSpan>1 and columnSpan>1:
	# 		# Translators: Speaks the row and column span added to the current row and column numbers
	# 		#			(example output: through row 5 column 3).
	# 		rowColSpanTranslation: str = _("through row {row} column {column}").format(
	# 			row=rowNumber + rowSpan - 1,
	# 			column=columnNumber + columnSpan - 1
	# 		)
	# 		textList.append(rowColSpanTranslation)
	rowCount=propertyValues.get('rowCount',0)
	columnCount=propertyValues.get('columnCount',0)
	if rowCount and columnCount:
		# Translators: Speaks number of columns and rows in a table (example output: with 3 rows and 2 columns).
		rowAndColCountTranslation: str = _("with {rowCount} rows and {columnCount} columns").format(
			rowCount=rowCount,
			columnCount=columnCount
		)
		textList.append(rowAndColCountTranslation)
	elif columnCount and not rowCount:
		# Translators: Speaks number of columns (example output: with 4 columns).
		columnCountTransation: str = _("with %s columns") % columnCount
		textList.append(columnCountTransation)
	elif rowCount and not columnCount:
		# Translators: Speaks number of rows (example output: with 2 rows).
		rowCountTranslation: str = _("with %s rows") % rowCount
		textList.append(rowCountTranslation)
	if rowCount or columnCount:
		# The caller is entering a table, so ensure that it is treated as a new table, even if the previous table was the same.
		_speechState.oldTableID = None

	# speak isCurrent property EG aria-current
	isCurrent = propertyValues.get('current', controlTypes.IsCurrent.NO)
	if isCurrent != controlTypes.IsCurrent.NO:
		textList.append(isCurrent.displayString)

	# are there further details
	hasDetails = propertyValues.get('hasDetails', False)
	if hasDetails:
		textList.append(
			# Translators: Speaks when there a further details/annotations that can be fetched manually.
			_("has details")
		)

	placeholder: Optional[str] = propertyValues.get('placeholder', None)
	if placeholder:
		textList.append(placeholder)
	indexInGroup=propertyValues.get('positionInfo_indexInGroup',0)
	similarItemsInGroup=propertyValues.get('positionInfo_similarItemsInGroup',0)
	if 0<indexInGroup<=similarItemsInGroup:
		# Translators: Spoken to indicate the position of an item in a group of items (such as a list).
		# {number} is replaced with the number of the item in the group.
		# {total} is replaced with the total number of items in the group.
		itemPosTranslation: str = _("{number} of {total}").format(
			number=indexInGroup,
			total=similarItemsInGroup
		)
		textList.append(itemPosTranslation)
	if 'positionInfo_level' in propertyValues:
		level=propertyValues.get('positionInfo_level',None)
		role=propertyValues.get('role',None)
		if level is not None:
			# Translators: Speaks the item level in treeviews (example output: level 2).
			levelTranslation: str = _('level %s') % level
			if (
				role in (controlTypes.Role.TREEVIEWITEM, controlTypes.Role.LISTITEM)
				and level != _speechState.oldTreeLevel
			):
				textList.insert(0, levelTranslation)
				_speechState.oldTreeLevel = level
			else:
				textList.append(levelTranslation)
	types.logBadSequenceTypes(textList)
	return textList




speech.speech.getObjectPropertiesSpeech = getObjectPropertiesSpeech

class GlobalPlugin(globalPluginHandler.GlobalPlugin):
	pass