

from ctypes import *
from ctypes.wintypes import *
import os
import wx # type: ignore
import addonHandler
import gui
import watchdog
from logHandler import log

# return codes from access4ulicense.h
a4u_success = 0
a4u_fail = 1
a4u_ts_tooOld = 101
a4u_ts_needsRecheck = 102
a4u_ts_valid = 103
a4u_ts_offline = 104
a4u_registryFail = 997
a4u_noValidHardwareID = 998
a4u_unknown = 999


prodID = 1094
prodName = "ExcelNavigatorPro"
regPath = "SOFTWARE\\ACCESS4U"

try:
	licDLL = cdll.licensingDLL
except:
	currentPath = os.path.abspath(os.path.dirname(__file__))
	licDLLPath = os.path.join(currentPath, "licensingDLL.dll")
	log.info(licDLLPath)
	licDLL = cdll.LoadLibrary(licDLLPath)	

getInstanceIDFuncType = CFUNCTYPE(c_int, c_wchar_p, POINTER(c_int), c_int, c_int, c_wchar_p )
_getInstanceID=getInstanceIDFuncType(('getInstanceID', licDLL),((1,'charBuffer'),(1 ,'bufferLength'), (1, 'variablesLength'), (1, 'PRODUCT_ID'), (1, 'hardwareID' )))

saveDataToRegistryFuncType = CFUNCTYPE(c_int, c_wchar_p, POINTER(c_int), c_wchar_p, c_wchar_p, c_wchar_p )
_saveDataToRegistry=saveDataToRegistryFuncType(('saveDataToRegistry', licDLL),((1,'charBuffer'),(1 ,'bufferLength'), (1, 'REG_PATH'), (1, 'licenseKey'), (1, 'acID' )))

#	__declspec(dllexport) int __cdecl activateProduct(wchar_t* charBuffer, int* bufferLength, int PRODUCT_ID, const wchar_t* REG_PATH, const wchar_t* PRODUCT_NAME, const wchar_t* LICENSE_KEY)
activateProductFuncType = CFUNCTYPE(c_int, c_wchar_p, POINTER(c_int), c_int, c_wchar_p, c_wchar_p, c_wchar_p )
_activateProduct=activateProductFuncType(('activateProduct', licDLL),((1,'charBuffer'),(1 ,'bufferLength'), (1, 'PRODUCT_ID'), (1, 'REG_PATH'), (1, 'PRODUCT_NAME'), (1, 'LICENSE_KEY' )))

# __declspec(dllexport) int __cdecl deactivateProduct(wchar_t* charBuffer, int* bufferLength, int PRODUCT_ID, const wchar_t* REG_PATH, const wchar_t* LICENSE_KEY)
deactivateProductFuncType = CFUNCTYPE(c_int, c_wchar_p, POINTER(c_int), c_int, c_wchar_p, c_wchar_p )
_deactivateProduct=deactivateProductFuncType(('deactivateProduct', licDLL),((1,'charBuffer'),(1 ,'bufferLength'), (1, 'PRODUCT_ID'), (1, 'REG_PATH'), (1, 'LICENSE_KEY' )))

# __declspec(dllexport) int __cdecl verifyActivationLocaly(wchar_t* charBuffer, int* bufferLength, LPCWSTR REG_PATH, LPCWSTR PRODUCT_NAME, int PRODUCT_ID)
verifyActivationLocalyFuncType = CFUNCTYPE(c_int, c_wchar_p, POINTER(c_int), c_wchar_p, c_wchar_p, c_int )
_verifyActivationLocaly=verifyActivationLocalyFuncType(('verifyActivationLocaly', licDLL),((1,'charBuffer'),(1 ,'bufferLength'), (1, 'REG_PATH'), (1, 'PRODUCT_NAME'), (1, 'PRODUCT_ID' )))


loadDataFromRegistryFuncType = CFUNCTYPE( 	 c_int,
											 c_wchar_p,
											 POINTER(c_int),
											 c_int,
											 c_wchar_p,
											 POINTER(c_uint64),
											 c_wchar_p,
											 c_wchar_p,
											 c_wchar_p,
											 c_int)
loadDataFromRegistryArgTypes = (   			
	( 1, 'charBuffer' ),
	( 1, 'bufferLength'),
	(1, 'variablesLength'),
	(1, 'licenseKeyRet'),
	(1, 'timeStampRet'),
	(1, 'hashRet'),
	( 1, 'REG_PATH'),
	( 1, 'PRODUCT_NAME'),
	( 1, 'PRODUCT_ID'),	
)

loadDataFromRegistry = loadDataFromRegistryFuncType(("loadDataFromRegistry", licDLL), loadDataFromRegistryArgTypes)


def activateProduct(licenseKey, PRODUCT_ID = prodID, REG_PATH =regPath, PRODUCT_NAME = prodName):
	charBuffer = c_wchar_p(2000 * " ")
	bufferLength = c_int(0)
	PRODUCT_ID = c_int(PRODUCT_ID)
	fullRegPath = os.path.join(REG_PATH, PRODUCT_NAME)
	REG_PATH = c_wchar_p(fullRegPath)
	PRODUCT_NAME = c_wchar_p(PRODUCT_NAME)
	licenseKey = c_wchar_p(licenseKey)

	res = _activateProduct(charBuffer ,byref(bufferLength), PRODUCT_ID, REG_PATH, PRODUCT_NAME, licenseKey)
	log.info(f"activateProduct: {res}")
	return res

def deactivateProduct(PRODUCT_ID = prodID, REG_PATH =regPath, PRODUCT_NAME = prodName):
	charBuffer = c_wchar_p(2000 * " ")
	bufferLength = c_int(0)
	PRODUCT_ID = c_int(PRODUCT_ID)
	fullRegPath = os.path.join(REG_PATH, PRODUCT_NAME)
	REG_PATH = c_wchar_p(fullRegPath)
	PRODUCT_NAME = c_wchar_p(PRODUCT_NAME)

	licenseKeyRet = c_wchar_p(50 * " ")
	hashRet = c_wchar_p(50 * " ")
	timeRet = c_uint64()
	variablesLength = c_int(50)
	log.info("deactivating product")
	res =loadDataFromRegistry(
		charBuffer, 
		byref(bufferLength), 
		variablesLength, 
		licenseKeyRet, 
		timeRet, 
		hashRet, 
		REG_PATH, 
		PRODUCT_NAME, 
		PRODUCT_ID  
	)
	
	res = _deactivateProduct(charBuffer ,byref(bufferLength), PRODUCT_ID, REG_PATH, licenseKeyRet)
	log.info(f"deactivateProduct: {res}")
	return res

def verifyActivationLocaly(PRODUCT_ID = prodID, REG_PATH =regPath, PRODUCT_NAME = prodName):
	charBuffer = c_wchar_p(2000 * " ")
	bufferLength = c_int(0)
	PRODUCT_ID = c_int(PRODUCT_ID)
	fullRegPath = os.path.join(REG_PATH, PRODUCT_NAME)
	REG_PATH = c_wchar_p(fullRegPath)
	PRODUCT_NAME = c_wchar_p(PRODUCT_NAME)
	log.info("verifying activation")
	res = _verifyActivationLocaly(charBuffer ,byref(bufferLength), REG_PATH, PRODUCT_NAME, PRODUCT_ID)
	log.info(f"verify result: {res}")
	return res

licenseVerified = False
def isLicensed():
	log.info("checking license")
	global licenseVerified
	if licenseVerified:
		return True
	res = verifyActivationLocaly()
	if res == a4u_success:
		licenseVerified = True
		log.info("license verified")
		return True
	if res == a4u_fail:
		log.info("license not valid")
		return False	
	log.info("license not verified")
	return True

def getInstanceID(productID):
	charBuffer = c_wchar_p(2000 * " ")
	bufferLength = c_int(0)
	variablesLength = 50
	instanceID = c_wchar_p(300 * " ")
	res = _getInstanceID(charBuffer ,byref(bufferLength), variablesLength, productID, instanceID)
	return instanceID.value

def saveDataToRegistry(lic, acID, REG_PATH):
	tmp = c_wchar_p(20000 * " ")
	REG_PATH = c_wchar_p(REG_PATH)
	acID = c_wchar_p (acID)
	lic = c_wchar_p (lic)

	bufferLength = c_int(2000)
	# p_acID = c_void_p.from_buffer(acID)
	# p_regPath = c_void_p.from_buffer(REG_PATH)
	# p_lic = c_void_p.from_buffer(lic)

	res = _saveDataToRegistry(tmp, byref(bufferLength), REG_PATH, lic, acID)
	log.info("saved to registry")
	return

